local Mounted = Mounted
local L = LibStub("AceLocale-3.0"):GetLocale("Mounted", false)

Mounted.DescOption = {
	type="group",
	width = 'fill',
	handler = Mounted,
	childGroups = 'tab',
	args={
		cm = {
			name = L["Combat"],
			desc = L["This Macro will be used in Combat and Indoors!"],
			type="group",
			order = 1,
			args={
			},
		},
		f = {
			name = L['Flight'],
			desc = L['This Macro will be used in any location you may fly!'],
			type="group",
			order = 2,
			args={
			},
		},
		g = {
			name = L['Ground'],
			desc = L['This Macro will be used in any location you may ride!'],
			type="group",
			order = 3,
			args={
			},
		},
		aq = {
			name = L["Ahn'Qiraj"],
			desc = L["This Macro will be used in Ahn'Qiraj"],
			type="group",
			order = 4,
			args={
			},
		},
	},
}

local db = {
	["char"] = L['char'],
	["class"] = L['class'],
	["race"] = L['race'],
	['global'] = L['global'],
}

local MacroOptions = {
	select={
		name=L['Select'],
		type="select",
		order = 10,
		values = function(info)
			local v = {}
			for k in pairs(Mounted.db.global.macros[info[#info-1]]) do
				if k == 'Default' or k == "Default Dismount" then
					v[k] = L[k]
				else
					v[k] = k
				end
			end
			v['new'] = L["New"]
			return v
		end,
		get = 'OGet',
		set = 'OSet',
	},
	name={
		name=function(info)
			return Mounted:OGet(info)
		end,
		type="header",
		order = 20,
	},
	macro={
		name=function(info)
			return Mounted:OGet(info)
		end,
		type="description",
		order = 30,
	},
	edit = {
		name=L['Edit Macro'],
		desc=L['Click to edit current macro.'],
		type="execute",
		order = 35,
		func = 'Ofunc',
		hidden = 'ODisabled',
		disabled = 'ODisabled',
	},
	delete = {
		name=L['Delete Macro'],
		desc=L['Will delete the selected macro and revert to the default macro.'],
		type="execute",
		order = 40,
		func = 'Ofunc',
		hidden = 'ODisabled',
		disabled = 'ODisabled',
	},
	preview={
		type = 'toggle',
		order = 50,
		name = L['Preview'],
		desc = L['Toggle Preview'],
		get = 'OGet',
		set = 'OSet',
	},
	linebreak = {
		name = '',
		order = 80,
		type="description",
	},
	db={
		name=L['db'],
		type="select",
		order = 90,
		values = db,
		get = 'OGet',
		set = 'OSet',
	},
}

Mounted.f = {}
Mounted.g = {}
Mounted.aq = {}
Mounted.cm = {}

function Mounted:OGet(info)
	if info[#info] == 'select' or info[#info] == 'name' then
		return self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]]
	elseif info[#info] == 'db' then
		return self.db.char[info[#info-1]..'loc']
	elseif info[#info] == 'macro' then
		if self[info[#info-1]].preview then
			return self:ParseMacro(self.db.global.macros[info[#info-1]][self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]]], true)
		else
			return self.db.global.macros[info[#info-1]][self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]]]
		end
	elseif info[#info] == 'preview' then
		return self[info[#info-1]].preview
	end
end

function Mounted:OSet(info, key)
	if info[#info] == 'select' then
		if key == 'new' then
			self:OpenMacroEditor(info[#info-1])
		else
			self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]] = key
		end
	elseif info[#info] == 'db' then
		if not self.db[key][info[#info-1]] then
			self.db[key][info[#info-1]] = self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]]
		end
		self.db.char[info[#info-1]..'loc'] = key
	elseif info[#info] == 'preview' then
		self[info[#info-1]].preview = key
	end
end

function Mounted:Ofunc(info)
	if info[#info] == 'delete' then
		self.db.global.macros[info[#info-1]][self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]]] = nil
		self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]] = 'Default'
		self:SelectMacro(true)
	elseif info[#info] == 'edit' then
		self:OpenMacroEditor(info[#info-1], true)
	end
end

function Mounted:Ovalidate(info, macro)
	for i=1,10 do
		if string.len(self:ParseMacro(macro, true)) > 255 then
			self:Print(L["Macro length error"])
			return L["Macro length error"]
		end
	end
	return true
end

function Mounted:ODisabled(info)
	if info[#info] == 'delete' or info[#info] == 'edit'then
		if self.locked[info[#info-1]][self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]]] then
			return true
		end
	end
	return nil
end

Mounted.DescOption.args.g.args = MacroOptions
Mounted.DescOption.args.f.args = MacroOptions
Mounted.DescOption.args.aq.args = MacroOptions
Mounted.DescOption.args.cm.args = MacroOptions

function Mounted:UpdateOptions()
	self:RangeCategory('f', '310')
	self:RangeCategory('f', '280')
	self:RangeCategory('f', '60')
	self:RangeCategory('g', '100')
	self:RangeCategory('g', '60')
	self:RangeCategory('g', 'aq')
end

local Rangetables = {}
function Mounted:RangeCategory(mtype, mspeed)
	local mounts = {}
	local optionname = string.format(L["%s: %s"], L[mtype], L[mspeed])
	local Registered = false
	if Rangetables[optionname] then
		Registered = true
		Rangetables[optionname].args = {}
	else
		Rangetables[optionname] = {	
			type="group",
			width = 'fill',
			handler = Mounted,
			args={
			},
		}
	end
	for k in pairs(self.db.char.mounts[mtype][mspeed]) do
		mounts[#mounts + 1] = GetSpellInfo(k)
	end
	table.sort(mounts)
	for i, mountname in ipairs(mounts) do
		Rangetables[optionname].args[string.format('%d_%s_%s', i, mtype, mspeed)] = self:CreateRange(mountname, i)
	end
	if next(Rangetables[optionname].args) and not Registered then
		self:RegisterModuleOptions(optionname, Rangetables[optionname], optionname)
	elseif not Registered then
		Rangetables[optionname] = nil
	end
end

function Mounted:GetRange(info, info2)
	local mtype, mspeed = string.match(info[#info], '%d+_(.)_(.+)')
	return self.db.char.mounts[mtype][mspeed][self.ReverseLookup[info.option.name]]
end

function Mounted:SetRange(info, num)
	local mtype, mspeed = string.match(info[#info], '%d+_(.)_(.+)')
	self.db.char.mounts[mtype][mspeed][self.ReverseLookup[info.option.name]] = num
end

function Mounted:CreateRange(mountname,ordernum)
	local Range = {
			type = 'range',
			name = mountname,
			order = ordernum,
			min = 0,
			max = 10,
			step = 1,
			get = 'GetRange',
			set = 'SetRange',
		}
	return Range
end

Mounted.optframe = {}
LibStub("AceConfig-3.0"):RegisterOptionsTable('Mounted', Mounted.DescOption)
Mounted.optframe.Mounted = LibStub("AceConfigDialog-3.0"):AddToBlizOptions('Mounted', 'Mounted')
-- function addapted from the RegisterModuleOptions function in Mapster
function Mounted:RegisterModuleOptions(name, optionTbl, displayName)
	local cname = "Mounted"..name
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(cname, optionTbl)
	self.optframe[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(cname, displayName, "Mounted")
end

--/script Mounted:OpenMacroEditor('f', macrotext)
do
	local MacroFrame
	local currmacrotext
	local macrotype
	local preview = false
	local function SetTooltip(frame)
		GameTooltip:ClearLines()
		GameTooltip:SetOwner(frame, 'ANCHOR_BOTTOMRIGHT', -305,20)
		GameTooltip:AddLine(L["FAQheader"])
		GameTooltip:AddLine("|cffffffff"..L["FAQdesc"])
		GameTooltip:SetToplevel(true)
		GameTooltip:Show()
	end
	function Mounted:OpenMacroEditor(mtype, edit)
		macrotype = mtype
		if MacroFrame then
			MacroFrame:Show()
		else
			local FrameBackdrop = {
					bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
					edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
					tile = true, tileSize = 32, edgeSize = 32, 
					insets = { left = 8, right = 8, top = 8, bottom = 8 }
				}
			local ControlBackdrop = {
					bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
					edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
					tile = true, tileSize = 16, edgeSize = 16,
					insets = { left = 3, right = 3, top = 3, bottom = 3 }
				}
			MacroFrame = CreateFrame("Frame","Mounted_Macro_Edit_Frame",UIParent)
				MacroFrame:SetFrameStrata("TOOLTIP")
				MacroFrame:SetWidth(350)
				MacroFrame:SetHeight(243)
				MacroFrame:SetPoint("CENTER",UIParent,"CENTER",0,0)
				MacroFrame:SetBackdrop(FrameBackdrop)
				MacroFrame:SetBackdropColor(0,0,0,1)
				MacroFrame:SetMovable(true)
			local SaveButton = CreateFrame("Button",nil,MacroFrame,"UIPanelButtonTemplate")
				SaveButton:SetHeight(24)
				SaveButton:SetWidth(100)
				local SaveText = SaveButton:GetFontString()
					SaveText:SetPoint("LEFT",SaveButton,"LEFT",15,0)
					SaveText:SetPoint("RIGHT",SaveButton,"RIGHT",-15,0)
					SaveText:SetText('Save & Set')
				SaveButton:SetPoint("BOTTOMLEFT",MacroFrame,"BOTTOMLEFT",15,15)
			local CloseButton = CreateFrame("Button",nil,MacroFrame,"UIPanelButtonTemplate")
				CloseButton:SetHeight(24)
				CloseButton:SetWidth(100)
				local CloseText = CloseButton:GetFontString()
					CloseText:SetPoint("LEFT",CloseButton,"LEFT",15,0)
					CloseText:SetPoint("RIGHT",CloseButton,"RIGHT",-15,0)
					CloseText:SetText(CANCEL)
				CloseButton:SetPoint("BOTTOMRIGHT",MacroFrame,"BOTTOMRIGHT",-15,15)
			local PreviewButton = CreateFrame("Button",nil,MacroFrame,"UIPanelButtonTemplate")
				PreviewButton:SetHeight(24)
				PreviewButton:SetWidth(100)
				local PreviewText = PreviewButton:GetFontString()
					PreviewText:SetPoint("LEFT",PreviewButton,"LEFT",15,0)
					PreviewText:SetPoint("RIGHT",PreviewButton,"RIGHT",-15,0)
					PreviewText:SetText(L['Preview'])
				PreviewButton:SetPoint("BOTTOM",MacroFrame,"BOTTOM",0,15)
			local MacroEditBox = CreateFrame("Frame", nil, MacroFrame)
				MacroEditBox:SetHeight(150)
				MacroEditBox:SetPoint("BOTTOMLEFT",SaveButton,"TOPLEFT",0,0)
				MacroEditBox:SetPoint("BOTTOMRIGHT",CloseButton,"TOPRIGHT",0,0)
				MacroEditBox:EnableMouse(true)
				local MacroEditBoxbackdrop = CreateFrame("Frame", nil, MacroEditBox)
					MacroEditBoxbackdrop:SetBackdrop(ControlBackdrop)
					MacroEditBoxbackdrop:SetBackdropColor(0, 0, 0)
					MacroEditBoxbackdrop:SetBackdropBorderColor(0.4, 0.4, 0.4)
					MacroEditBoxbackdrop:SetPoint("TOPLEFT",MacroEditBox,"TOPLEFT",0, -20)
					MacroEditBoxbackdrop:SetPoint("BOTTOMRIGHT",MacroEditBox,"BOTTOMRIGHT",0,0)
					MacroEditBoxbackdrop:SetScript("OnEnter",function() SetTooltip(MacroEditBoxbackdrop) end)
					MacroEditBoxbackdrop:SetScript("OnLeave",function() GameTooltip:Hide() end)
				local MacroEditScrollframe = CreateFrame("ScrollFrame", 'Mounted_MacroEditScrollframe', MacroEditBoxbackdrop, "UIPanelScrollFrameTemplate")
					MacroEditScrollframe:SetPoint("TOPLEFT", 5, -6)
					MacroEditScrollframe:SetPoint("BOTTOMRIGHT", -28, 6)
					MacroEditScrollframe:SetScript("OnEnter",function() SetTooltip(MacroEditBoxbackdrop) end)
					MacroEditScrollframe:SetScript("OnLeave",function() GameTooltip:Hide() end)
				local MacroEditScrollchild = CreateFrame("Frame", nil, MacroEditScrollframe)
					MacroEditScrollframe:SetScrollChild(MacroEditScrollchild)
					MacroEditScrollchild:SetHeight(MacroEditScrollframe:GetHeight())
					MacroEditScrollchild:SetWidth(MacroEditScrollframe:GetWidth())
					MacroEditScrollchild:SetScript("OnEnter",function() SetTooltip(MacroEditBoxbackdrop) end)
					MacroEditScrollchild:SetScript("OnLeave",function() GameTooltip:Hide() end)
				local MacroEditLabel = MacroEditBox:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
					MacroEditLabel:SetPoint("TOPLEFT",MacroEditBox,"TOPLEFT",0,-2)
					MacroEditLabel:SetPoint("TOPRIGHT",MacroEditBox,"TOPRIGHT",0,-2)
					MacroEditLabel:SetJustifyH("LEFT")
					MacroEditLabel:SetHeight(18)
					MacroEditLabel:SetText(ENTER_MACRO_LABEL)
				local MacroEditEditbox = CreateFrame("EditBox", nil, MacroEditScrollchild)
					MacroEditEditbox:SetPoint("TOPLEFT")
					MacroEditEditbox:SetHeight(50)
					MacroEditEditbox:SetWidth(MacroEditScrollframe:GetWidth())
					MacroEditEditbox:SetMultiLine(true)
					MacroEditEditbox:SetTextInsets(5, 5, 3, 3)
					MacroEditEditbox:EnableMouse(true)
					MacroEditEditbox:SetAutoFocus(false)
					MacroEditEditbox:SetFontObject(ChatFontNormal)
					MacroEditEditbox:SetScript("OnEscapePressed",function() MacroEditEditbox:ClearFocus() end)
					MacroEditEditbox:SetScript("OnEnter",function() SetTooltip(MacroEditBoxbackdrop) end)
					MacroEditEditbox:SetScript("OnLeave",function() GameTooltip:Hide() end)
				MacroEditScrollframe:EnableMouse(true)
				MacroEditScrollframe:SetScript("OnMouseUp", function() MacroEditEditbox:SetFocus() end)
			local MacroNameBox = CreateFrame("Frame", nil, MacroFrame)
				MacroNameBox:SetHeight(44)
				MacroNameBox:SetPoint("BOTTOMLEFT",MacroEditBox,"TOPLEFT",0,0)
				MacroNameBox:SetPoint("BOTTOMRIGHT",MacroEditBox,"TOPRIGHT",0,0)
				local MacroNameEditbox = CreateFrame("EditBox","Mounted_MacroNameBox",MacroNameBox,"InputBoxTemplate")
					MacroNameEditbox:SetFontObject(ChatFontNormal)
					MacroNameEditbox:SetTextInsets(0,0,3,3)
					MacroNameEditbox:SetMaxLetters(16)
					MacroNameEditbox:SetPoint("BOTTOMLEFT",MacroNameBox,"BOTTOMLEFT",6,0)
					MacroNameEditbox:SetPoint("BOTTOMRIGHT",MacroNameBox,"BOTTOMRIGHT",0,0)
					MacroNameEditbox:SetHeight(19)
					MacroNameEditbox:EnableMouse(true)
					MacroNameEditbox:SetAutoFocus(false)
					MacroNameEditbox:SetScript("OnEscapePressed",function() MacroNameEditbox:ClearFocus() end)
				local MacroNameLabel = MacroNameBox:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
					MacroNameLabel:SetPoint("TOPLEFT",MacroNameBox,"TOPLEFT",0,-2)
					MacroNameLabel:SetPoint("TOPRIGHT",MacroNameBox,"TOPRIGHT",0,-2)
					MacroNameLabel:SetJustifyH("LEFT")
					MacroNameLabel:SetHeight(18)
					MacroNameLabel:SetText(MACRO_POPUP_TEXT)
			CloseButton:SetScript("OnClick",function()
					MacroEditEditbox:SetText('')
					MacroNameEditbox:SetText('')
					currmacrotext = nil
					preview = false
					MacroFrame:Hide()
				end)
			PreviewButton:SetScript("OnClick",function()
					MacroEditEditbox:ClearFocus()
					MacroNameEditbox:ClearFocus()
					if not preview then
						currmacrotext = MacroEditEditbox:GetText()
					end
					MacroEditEditbox:SetText(Mounted:ParseMacro(currmacrotext, true))
					preview = true
				end)
			MacroEditEditbox:SetScript("OnEditFocusGained",function()
					if currmacrotext then
						MacroEditEditbox:SetText(currmacrotext)
						currmacrotext = nil
					end
					preview = false
				end)
			SaveButton:SetScript("OnClick",function()
					if not MacroNameEditbox:GetText() or MacroNameEditbox:GetText() == '' or Mounted.locked[macrotype][currmacrotext or MacroNameEditbox:GetText()] then
						return Mounted:Print(string.format(L['NameError'], MacroNameEditbox:GetText() or ''))
					end
					local macrotext = currmacrotext or MacroEditEditbox:GetText()
					if macrotext and string.gsub(macrotext, ' ', '') ~= '' then
						for i=1,10 do
							if string.len(Mounted:ParseMacro(macrotext, true)) > 255 then
								return Mounted:Print(L["Macro length error"])
							end
						end
					else
						return Mounted:Print(L["No Macro Text"])
					end
					Mounted.db.global.macros[macrotype][MacroNameEditbox:GetText()] = macrotext
					Mounted.db[Mounted.db.char[macrotype..'loc']][macrotype] = MacroNameEditbox:GetText()
					currmacrotext = nil
					preview = false
					Mounted:SelectMacro(true)
					LibStub("AceConfigDialog-3.0"):ConfigTableChanged(nil,'Mounted')
					MacroFrame:Hide()
				end)
			MacroFrame.macro = MacroEditEditbox
			MacroFrame.name = MacroNameEditbox
			tinsert(UISpecialFrames, "Mounted_Macro_Edit_Frame")
		end
		if edit then
			MacroFrame.name:SetText(Mounted.db[Mounted.db.char[macrotype..'loc']][macrotype])
		else
			MacroFrame.name:SetText('')
		end
		MacroFrame.macro:SetText(Mounted.db.global.macros[macrotype][Mounted.db[Mounted.db.char[macrotype..'loc']][macrotype]])
	end
end
